==> unicore-ucc6-5.0.0-1.src.tar.gz: ucc-ccm/src/main/java/eu/unicore/ucc/ccm/CCMAction.java <==
/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 10-05-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package eu.unicore.ucc.ccm;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Date;

import javax.net.ssl.SSLHandshakeException;

import jline.ConsoleReader;

import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.io.FileUtils;
import org.apache.xmlbeans.XmlException;

import xmlbeans.eu.unicore.ccm.internal.ClientProfileDocument.ClientProfile;

import de.fzj.unicore.ucc.Command;
import de.fzj.unicore.ucc.MessageWriter;
import eu.unicore.ccm.CCMManager;

public class CCMAction extends Command
{
	public static final String OPT_NONINTERACTIVE_LONG="assumeyes";
	public static final String OPT_NONINTERACTIVE="b";

	public static final String OPT_UPDATE_LONG="updateexisting";
	public static final String OPT_UPDATE="u";

	public String getName()
	{
		return "download-config";
	}

	@Override
	@SuppressWarnings("all")
	protected void createOptions() 
	{
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_PROPERTIES_LONG)
				.withDescription("Properties file containing your preferences. By default, a file '<userhome>/.ucc/preferences' is checked.")
				.withArgName("Properties")
				.hasArg()
				.isRequired(false)
				.create(OPT_PROPERTIES)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_NONINTERACTIVE_LONG)
				.withDescription("Non interactive mode, all confirmations are automatically accepted")
				.withArgName("AssumeYes")
				.isRequired(false)
				.create(OPT_NONINTERACTIVE)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_UPDATE_LONG)
				.withDescription("Update the (currently used) configuration of the client. If this option is not selected then downloaded configuration will be saved in a separate directory.")
				.withArgName("UpdateExisting")
				.isRequired(false)
				.create(OPT_UPDATE)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_TRUSTSTORE_LONG)
				.withDescription("(optional)Truststore in jks format containing your trusted certificates")
				.withArgName("Truststore")
				.hasArg()
				.isRequired(false)
				.create(OPT_TRUSTSTORE)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_TRUSTSTOREPASSWORD_LONG)
				.withDescription("(optional)Truststore password")
				.withArgName("TrustPassword")
				.hasArg()
				.isRequired(false)
				.create(OPT_TRUSTSTOREPASSWORD)
				);
	}

	@Override
	public String getSynopsis()
	{
		return "Downloads a configuration profile for your client appliaction. " +
			"Such configuration profile may be published by your grid infrastructure " +
			"provider to quickly connect to the resouces. You must provide at least " +
			"a URL of the profile and a fingerprint of a server's certificate " +
			"which is hosting the profile.";
	}

	@Override
	public String getDescription()
	{
		return "downloads configuration profile for this client";
	}

	@Override
	public String getArgumentList()
	{
		return "<https profile URL> <server's certificate SHA1 fingerprint>";
	}

	@Override
	public String getCommandGroup()
	{
		return "General";
	}

	@Override
	public void process()
	{
		super.process();
		if (getCommandLine().getArgs().length < 3)
		{
			message("Please provide a profile's URL and a fingerprint arguments.");
			printUsage();
			endProcessing(ERROR_CLIENT);
		}
		String address = getCommandLine().getArgs()[1];
		String fingerprint = getCommandLine().getArgs()[2];

		if (propertiesFile == null)
		{
			error("Properties file is undefined", null);
			endProcessing(ERROR_CLIENT);
		}
		
		CCMManager ccmManager = new CCMManager();
		ClientProfile profile = null;
		try
		{
			profile = ccmManager.download(address, fingerprint);
		} catch (SSLHandshakeException e)
		{
			error("The profile can not be downloaded from the server," +
					" as the SSL connection can not be verified: " 
					+ e.getMessage(), e);
			endProcessing(ERROR_CLIENT);
		} catch (IOException e)
		{
			error("There was a communication problem", e);
			endProcessing(ERROR_CLIENT);
		} catch (XmlException e)
		{
			error("Profile was downloaded but is invalid", e);
			endProcessing(ERROR_CLIENT);
		} catch (Exception e)
		{
			error("Error downloading a profile", e);
			endProcessing(ERROR_CLIENT);
		}
		
		boolean update = getBooleanOption(OPT_UPDATE_LONG, OPT_UPDATE);
		boolean assumeYes = getBooleanOption(OPT_NONINTERACTIVE_LONG, OPT_NONINTERACTIVE);
		
		confirmInstallation(profile, update, assumeYes);
		
		String targetDirName = null;
		if (!update)
		{
			String baseDir = output.getAbsolutePath() + File.separator + 
				"clientConfiguration-" + profile.getName();
			targetDirName = findUnusedName(baseDir);
			File targetDir = new File(targetDirName);
			if (!targetDir.mkdir())
			{
				error("Error creating the working directory " 
						+ targetDirName + " for the downloaded configuration.", null);
				endProcessing(ERROR_CLIENT);
			}
			verbose("Using " + targetDirName + " as a target directory");
		}

		PropertiesUpdater propertiesUtil = new PropertiesUpdater(this, targetDirName);
		PluginsUpdater pluginsUtil = new PluginsUpdater(this, targetDirName);
		TrustUpdater trustUtil = new TrustUpdater(this, targetDirName);
		try
		{	
			KeyStoreParameters tsParams = getCurrentTSWrapped();
			
			propertiesUtil.processRegistries(profile, propertiesFile.getAbsolutePath(), 
					update);
			trustUtil.processTrustedCAs(profile, tsParams, update);
			pluginsUtil.processPlugins(profile, PluginsUpdater.getMainExtensionsFile(), 
				update);
		} catch (CCMException e)
		{
			error(e.getMessage(), e.getCause());
			endProcessing(ERROR_CLIENT);
		}
		
		message("");
		if (update)
			message("The configuration profile '" + profile.getName() + 
				"' was successfully loaded. Note that in case of usage of " +
				"UCC shell you must restart the client to apply changes.");
		else
			message("The configuration profile '" + profile.getName() + 
				"' was successfully stored in the '" + targetDirName + 
				"' directory. You can review this configuration and" +
				" use it instead of your existing one.");
	}

	
	private void confirmInstallation(ClientProfile profile, boolean update, boolean assumeYes)
	{
		message(CCMManager.INSTRUCTION_INTRO);
		message("");
		message(CCMManager.INSTRUCTION_GENERIC);
		message("");
		message(CCMManager.INSTRUCTION_SECURITY);
		message("");
		if (update)
			message("The currently used configuation will be updated " +
					"after being backed up.");
		if (!assumeYes)
		{
			try{
				String line = new ConsoleReader().readLine("Are you sure that you want to continue? [N] ");
				if (!(line.startsWith("y") || line.startsWith("Y")))
				{
					message("Exiting on user's request.");
					endProcessing();
				}
			}catch(IOException e)
			{
				error("Can't read user's answer", e);
				endProcessing(ERROR_CLIENT);
			}
		}
	}
	
	private static String findUnusedName(String base)
	{
		String file = base;
		int i = 1;
		while (new File(file).exists())
			file = base + "." + i++;
		return file;
	}
	
	public static void backupFile(String original, MessageWriter writer) throws CCMException 
	{
		DateFormat dt = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
		String time = dt.format(new Date());
		time = time.replace(' ', '_');
		String backupBase = original + "_backup_" + time;
		
		String backupFile = findUnusedName(backupBase);
		File origF = new File(original);
		File targetF = new File(backupFile);
		try
		{
			FileUtils.copyFile(origF, targetF);
		} catch (IOException e)
		{
			throw new CCMException("Can't backup the " + original + " file", e);
		}
		if (writer != null)
			writer.message("Created a backup of the original file: " + original + 
				" named: " + backupFile);
	}

	private KeyStoreParameters getCurrentTSWrapped() throws CCMException
	{
		try
		{
			return getCurrentTS();
		} catch (IOException e)
		{
			throw new CCMException("Can't read the existing truststore configuration", e);
		}
	}
	
	private KeyStoreParameters getCurrentTS() throws IOException
	{
		String truststore = getOption(OPT_TRUSTSTORE_LONG, OPT_TRUSTSTORE, 
				properties.getProperty(OPT_TRUSTSTORE_LONG));
		String tsType = "jks";
		String tsPasswd = "";
		if (truststore != null)
		{
			truststore = makeRelative(truststore, OPT_TRUSTSTORE);
			tsPasswd = getOption(OPT_TRUSTSTOREPASSWORD_LONG, OPT_TRUSTSTOREPASSWORD);
			if (tsPasswd != null)
				verbose("Truststore password given.");
			else 
			{
				verbose("No truststore password given.");
				message("Please enter your truststore password:");
				tsPasswd = new jline.ConsoleReader().readLine(
						new Character('*'));
			}
		} else
		{
			String keystore = getOption(OPT_KEYSTORE_LONG, OPT_KEYSTORE);
			String ksType = "jks";
			String ksPasswd = null;
			if (keystore != null)
			{
				keystore = makeRelative(keystore, OPT_KEYSTORE);
				ksType = getOption(OPT_KEYSTORETYPE_LONG, OPT_KEYSTORETYPE, "jks");
				ksPasswd = getOption(OPT_PASSWORD_LONG, OPT_PASSWORD);
				if (ksPasswd != null)
					verbose("Keystore password given.");
				else 
				{
					verbose("No keystore password given.");
					message("Please enter your keystore password:");
					ksPasswd = new jline.ConsoleReader().readLine(
							new Character('*'));
				}
			}
			if (keystore != null)
			{
				//use keystore parameters
				truststore = keystore;
				tsType = ksType;
				tsPasswd = ksPasswd;
			} else
			{
				error("Neither truststore nor keystore is specified " +
					"on command line and in client configuration file. " +
					"You must define one of them so that trusted CA certificates " +
					"can be read from", null);
				endProcessing(ERROR_CLIENT);
			}
		}
		File truststoreFile = new File(truststore);
		if (!truststoreFile.exists() || !truststoreFile.canRead())
			throw new IOException("Truststore file does not exist or is not readable. " + 
					truststoreFile);
		return new KeyStoreParameters(truststore, tsType, tsPasswd.toCharArray());
	}

	/**
	 * Make argument relative to properties file, if not absolute and set in prop file
	 * @param file
	 * @return
	 */
	private String makeRelative(String fileName, String opt)
	{
		File file=new File(fileName);
		if (!file.isAbsolute() && !getCommandLine().hasOption(opt))
		{
			if (propertiesFile!=null && propertiesFile.exists())
				file = new File(propertiesFile.getAbsoluteFile().
						getParent(), fileName);
			return file.getAbsolutePath();
		}
		return fileName;
	}
}

==> unicore-ucc6-6.0.0-0.src.tar.gz: ucc-ccm/src/main/java/eu/unicore/ucc/ccm/CCMAction.java <==
/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 10-05-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package eu.unicore.ucc.ccm;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Date;

import javax.net.ssl.SSLHandshakeException;

import jline.ConsoleReader;

import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.io.FileUtils;
import org.apache.xmlbeans.XmlException;

import xmlbeans.eu.unicore.ccm.internal.ClientProfileDocument.ClientProfile;

import de.fzj.unicore.ucc.Command;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.ucc.authn.KeystoreAuthN;
import eu.unicore.ccm.CCMManager;

public class CCMAction extends Command
{
	public static final String OPT_NONINTERACTIVE_LONG="assumeyes";
	public static final String OPT_NONINTERACTIVE="b";

	public static final String OPT_UPDATE_LONG="updateexisting";
	public static final String OPT_UPDATE="u";

	public String getName()
	{
		return "download-config";
	}

	@Override
	@SuppressWarnings("all")
	protected void createOptions() 
	{
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_PROPERTIES_LONG)
				.withDescription("Properties file containing your preferences. By default, a file '<userhome>/.ucc/preferences' is checked.")
				.withArgName("Properties")
				.hasArg()
				.isRequired(false)
				.create(OPT_PROPERTIES)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_NONINTERACTIVE_LONG)
				.withDescription("Non interactive mode, all confirmations are automatically accepted")
				.withArgName("AssumeYes")
				.isRequired(false)
				.create(OPT_NONINTERACTIVE)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_UPDATE_LONG)
				.withDescription("Update the (currently used) configuration of the client. If this option is not selected then downloaded configuration will be saved in a separate directory.")
				.withArgName("UpdateExisting")
				.isRequired(false)
				.create(OPT_UPDATE)
				);
	}

	@Override
	public String getSynopsis()
	{
		return "Downloads a configuration profile for your client appliaction. " +
			"Such configuration profile may be published by your grid infrastructure " +
			"provider to quickly connect to the resouces. You must provide at least " +
			"a URL of the profile and a fingerprint of a server's certificate " +
			"which is hosting the profile.";
	}

	@Override
	public String getDescription()
	{
		return "downloads configuration profile for this client";
	}

	@Override
	public String getArgumentList()
	{
		return "<https profile URL> <server's certificate SHA1 fingerprint>";
	}

	@Override
	public String getCommandGroup()
	{
		return "General";
	}

	@Override
	public void process()
	{
		super.process();
		if (getCommandLine().getArgs().length < 3)
		{
			message("Please provide a profile's URL and a fingerprint arguments.");
			printUsage();
			endProcessing(ERROR_CLIENT);
		}
		String address = getCommandLine().getArgs()[1];
		String fingerprint = getCommandLine().getArgs()[2];

		if (propertiesFile == null)
		{
			error("Properties file is undefined", null);
			endProcessing(ERROR_CLIENT);
		}
		
		CCMManager ccmManager = new CCMManager();
		ClientProfile profile = null;
		try
		{
			profile = ccmManager.download(address, fingerprint);
		} catch (SSLHandshakeException e)
		{
			error("The profile can not be downloaded from the server," +
					" as the SSL connection can not be verified: " 
					+ e.getMessage(), e);
			endProcessing(ERROR_CLIENT);
		} catch (IOException e)
		{
			error("There was a communication problem", e);
			endProcessing(ERROR_CLIENT);
		} catch (XmlException e)
		{
			error("Profile was downloaded but is invalid", e);
			endProcessing(ERROR_CLIENT);
		} catch (Exception e)
		{
			error("Error downloading a profile", e);
			endProcessing(ERROR_CLIENT);
		}
		
		boolean update = getBooleanOption(OPT_UPDATE_LONG, OPT_UPDATE);
		boolean assumeYes = getBooleanOption(OPT_NONINTERACTIVE_LONG, OPT_NONINTERACTIVE);
		
		confirmInstallation(profile, update, assumeYes);
		
		String targetDirName = null;
		if (!update)
		{
			String baseDir = output.getAbsolutePath() + File.separator + 
				"clientConfiguration-" + profile.getName();
			targetDirName = findUnusedName(baseDir);
			File targetDir = new File(targetDirName);
			if (!targetDir.mkdir())
			{
				error("Error creating the working directory " 
						+ targetDirName + " for the downloaded configuration.", null);
				endProcessing(ERROR_CLIENT);
			}
			verbose("Using " + targetDirName + " as a target directory");
		}

		PropertiesUpdater propertiesUtil = new PropertiesUpdater(this, targetDirName);
		PluginsUpdater pluginsUtil = new PluginsUpdater(this, targetDirName);
		TrustUpdater trustUtil = new TrustUpdater(this, targetDirName);
		try
		{	
			checkAuthnMethod();
			propertiesUtil.processRegistries(profile, propertiesFile.getAbsolutePath(), 
					update);
			trustUtil.processTrustedCAs(profile, update, propertiesFile.getAbsolutePath());
			pluginsUtil.processPlugins(profile, PluginsUpdater.getMainExtensionsFile(), 
				update);
		} catch (CCMException e)
		{
			error(e.getMessage(), e.getCause());
			endProcessing(ERROR_CLIENT);
		}
		
		message("");
		if (update)
			message("The configuration profile '" + profile.getName() + 
				"' was successfully loaded. Note that in case of usage of " +
				"UCC shell you must restart the client to apply changes.");
		else
			message("The configuration profile '" + profile.getName() + 
				"' was successfully stored in the '" + targetDirName + 
				"' directory. You can review this configuration and" +
				" use it instead of your existing one.");
	}

	
	private void confirmInstallation(ClientProfile profile, boolean update, boolean assumeYes)
	{
		message(CCMManager.INSTRUCTION_INTRO);
		message("");
		message(CCMManager.INSTRUCTION_GENERIC);
		message("");
		message(CCMManager.INSTRUCTION_SECURITY);
		message("");
		if (update)
			message("The currently used configuation will be updated " +
					"after being backed up.");
		if (!assumeYes)
		{
			try{
				String line = new ConsoleReader().readLine("Are you sure that you want to continue? [N] ");
				if (!(line.startsWith("y") || line.startsWith("Y")))
				{
					message("Exiting on user's request.");
					endProcessing();
				}
			}catch(IOException e)
			{
				error("Can't read user's answer", e);
				endProcessing(ERROR_CLIENT);
			}
		}
	}
	
	private static String findUnusedName(String base)
	{
		String file = base;
		int i = 1;
		while (new File(file).exists())
			file = base + "." + i++;
		return file;
	}
	
	public static void backupFile(String original, MessageWriter writer) throws CCMException 
	{
		DateFormat dt = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
		String time = dt.format(new Date());
		time = time.replace(' ', '_');
		String backupBase = original + "_backup_" + time;
		
		String backupFile = findUnusedName(backupBase);
		File origF = new File(original);
		File targetF = new File(backupFile);
		try
		{
			FileUtils.copyFile(origF, targetF);
		} catch (IOException e)
		{
			throw new CCMException("Can't backup the " + original + " file", e);
		}
		if (writer != null)
			writer.message("Created a backup of the original file: " + original + 
				" named: " + backupFile);
	}

	private void checkAuthnMethod() throws CCMException
	{
		String authNMethod = getOption(OPT_AUTHN_METHOD_LONG, OPT_AUTHN_METHOD, KeystoreAuthN.X509);
		if (!authNMethod.equals(KeystoreAuthN.X509))
			throw new CCMException("Configuration setup can be performed only when the default " + 
					KeystoreAuthN.X509 + " authentication method is used.");
	}
}
